require 'csv'
require 'active_support/inflector'
require 'date'

#################################################
# REMINDER: An empty cell can return nil or ''  #
#################################################

class Distributator < CSV
  def initialize(data, options = Hash.new)
    options.merge!({ headers: true,
                     header_converters: [self.class.rename_header,
                      *options[:header_converters]] })
    super(data, options)
  end

  def self.rename_header
    ->(header) do
      case header
      when /\A['"]?estimated_availability_date['"]?\Z/i
        'available_on'
      when /\A['"]?image_(\w+)_(\d+)['"]?\Z/
        # TODO: should this have a default value?
        lookup = { 'reference' => 'url', 'name' => 'title' }
        "images.#{$2}.#{lookup[$1]}"
      when /\A['"]?upc['"]?\Z/i
        'id'
      when /\A['"]?cost['"]?\Z/i
        'cost_price'
      when /\A['"]?product_title['"]?\Z/i
        'name'
      else
        header.gsub(/'|"/, '')
      end
    end
  end
end

module DSCO_Table
  class DSCOError < StandardError; end
  class PriceError < DSCOError; end

  def convert_attributes
    each do |row|
      attr_indices.each do |index|
        attr_name = "attribute_name_#{index}"
        attr_val = "attribute_value_#{index}"
        row["properties.#{row[attr_name]}"] = row[attr_val]
      end
    end

    attr_indices.each do |index|
      delete("attribute_name_#{index}")
      delete("attribute_value_#{index}")
    end; return nil
  end

  def convert_price
    each do |row|
      map, msrp = row['map'], row['msrp']

      price = map unless map.nil? || map.to_f.zero? # implies #empty?
      price ||= msrp
      raise PriceError if price.to_f.zero?
      row['price'] = price
    end

    delete('msrp'); delete('map')
  end

  def convert_taxons
    each do |row|
      row['category'].split('|').each_with_index do |taxon, i|
        # WARNING: Jenson only uses one taxon group per product
        # So this code only handles one taxon group.
        row["taxons.0.#{i}"] = taxon
      end
    end
    delete('category')
  end

  def make_permalinks
    each do |row|
      row['permalink'] = row['name'].parameterize
    end
  end

  def normalize_available_on
    each do |row|
      if row['available_on'].nil? || row['available_on'].empty?
        row['available_on'] = DateTime.now.iso8601
      end
    end
  end

  private
  def attr_indices
    headers.map { |h| h.match(/attribute_name_(\d+)/); $1 }.compact
  end
end

CSV::Table.include DSCO_Table
